@@ -1,5 +1,6 @@
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
+from rx.core.typing import Observable
 
 from typing import (
     Dict,
@@ -121,50 +122,41 @@ class Camera(BaseCamera):
 
 
 # camera types
-@dataclass
-class BaseState:
+class BaseState(TypedDict):
     timestamp: int
     lastTriggered: int
 
 
-@dataclass
-class MotionSetEvent:
+class MotionSetEvent(TypedDict):
     state: bool
 
 
-@dataclass
-class AudioSetEvent:
+class AudioSetEvent(TypedDict):
     state: bool
 
 
-@dataclass
-class ObjectSetEvent:
+class ObjectSetEvent(TypedDict):
     state: bool
     detections: List[str]
 
 
-@dataclass
-class LightSetEvent:
+class LightSetEvent(TypedDict):
     state: bool
 
 
-@dataclass
-class DoorbellSetEvent:
+class DoorbellSetEvent(TypedDict):
     state: bool
 
 
-@dataclass
-class SirenSetEvent:
+class SirenSetEvent(TypedDict):
     state: bool
 
 
-@dataclass
-class BatterySetEvent:
+class BatterySetEvent(TypedDict):
     state: int
 
 
-@dataclass
-class DeviceSetEvent:
+class DeviceSetEvent(TypedDict):
     pass
 
 
@@ -257,6 +249,10 @@ OnSetEvent = Union[
     ObjectSetEvent,
 ]
 
+StateNames = Literal[
+    "light", "motion", "audio", "doorbell", "siren", "battery", "object"
+]
+
 OnChangeCallback = Callable[[T, T], None]
 
 
@@ -313,7 +309,7 @@ CameraExtension = Literal[
 ]
 
 
-class BaseCameraConfig(TypedDict, total=False):
+class BaseCameraConfig(TypedDict):
     name: str
     nativeId: Optional[str]
     isCloud: Optional[bool]
@@ -325,7 +321,7 @@ class BaseCameraConfig(TypedDict, total=False):
     info: Optional[CameraInformation]
 
 
-class CameraInputSettings(TypedDict, total=False):
+class CameraInputSettings(TypedDict):
     _id: str
     name: str
     roles: List[CameraRoles]
@@ -485,6 +481,12 @@ class SchemaConfig(TypedDict):
     config: Dict[str, Union[str, int, bool]]
 
 
+class IceServer(TypedDict):
+    urls: List[str]
+    username: Optional[str]
+    credential: Optional[str]
+
+
 # camera storage
 @runtime_checkable
 class CameraStorage(Protocol):
@@ -619,8 +621,70 @@ class PluginsManager(Protocol):  # todo: add methods
 
 
 @runtime_checkable
-class CameraDevice(Protocol):  # todo
-    pass
+class CameraDevice(Protocol):
+    id: str
+    plugin_id: str
+    native_id: Optional[str]
+    state: bool
+    disabled: bool
+    name: str
+    type: CameraType
+    info: CameraInformation
+    sources: List[CameraSource]
+    motion_zones: List[CameraZone]
+    object_zones: List[CameraZone]
+    is_cloud: bool
+    has_light: bool
+    has_siren: bool
+    has_binary_sensor: bool
+    has_battery: bool
+    has_motion_detector: bool
+    has_audio_detector: bool
+    has_object_detector: bool
+    has_ptz: bool
+    on_started: Observable
+    on_light_switched: Observable
+    on_motion_detected: Observable
+    on_audio_detected: Observable
+    on_object_detected: Observable
+    on_doorbell_pressed: Observable
+    on_siren_detected: Observable
+    on_battery_changed: Observable
+    stream_source: CameraSource
+    snapshot_source: CameraSource
+    recording_source: CameraSource
+    detection_source: CameraSource
+    delegate: Optional[CameraDelegate]
+    prebuffer_delegate: Optional[CameraPrebufferDelegate]
+
+    async def connect(self) -> None: ...
+    async def disconnect(self) -> None: ...
+    async def reboot(self) -> None: ...
+    async def snapshot(self, force_new: Optional[bool] = None) -> bytes: ...
+    async def get_ffmpeg_path(self) -> str: ...
+    async def get_ice_servers(self) -> List[IceServer]: ...
+    # async def create_session(self, options: Any) -> Any: ... # todo
+    async def stream_video(self, source_name: str, options: Any) -> None: ...
+    async def record_to_file(
+        self, source_name: str, output_path: str, duration: Optional[int] = 60
+    ) -> None: ...
+    async def get_value(
+        self, state_name: str
+    ) -> (
+        LightState
+        | MotionState
+        | AudioState
+        | ObjectState
+        | DoorbellState
+        | SirenState
+        | BatteryState
+    ): ...
+    async def update_state(
+        self, state_name: str, event_data: Any, is_remote: Optional[bool] = None
+    ) -> None: ...
+    def on_state_change(self, state_name: StateNames) -> Observable: ...
+    def on_property_change(self, property_name: str) -> Observable: ...
+    def remove_all_listeners(self) -> None: ...
 
 
 # device manager
